--- usr.bin/passwd/local_passwd.c.orig	Sat May 29 00:02:38 2004
+++ usr.bin/passwd/local_passwd.c	Sat May 29 00:22:28 2004
@@ -67,6 +67,10 @@
 #include <login_cap.h>
 #endif
 
+#ifndef CRACKLIB_DICTPATH /* For Alec Muffet's Cracklib */ 
+#define CRACKLIB_DICTPATH "/usr/local/libdata/cracklib/pw_dict"
+#endif
+
 #include "extern.h"
 
 static uid_t uid;
@@ -90,9 +94,10 @@
 }
 
 char *
-getnewpasswd(pw, nis)
+getnewpasswd(pw, nis, oldpasswd)
 	struct passwd *pw;
 	int nis;
+	const char *oldpasswd;
 {
 	int tries, min_length = 6;
 	int force_mix_case = 1;
@@ -145,7 +150,37 @@
 			(void)printf("Please enter a password at least %d characters in length.\n", min_length);
 			continue;
 		}
+		/* the same? */
+		{
+			int same = 0, i;
+			for (t = p; *t; t++) {
+				if (strchr(oldpasswd, *t))
+					same++;
+			}
+			if (same >= 3) {
+				(void)printf("Please do not reuse the same characters in old password.\n");
+				continue;
+			}
+		}
 		
+/* cracklib start */
+#ifdef CRACKLIB_DICTPATH
+                {       
+                        char *msg;
+                        if (msg = (char *) (FascistCheck(p, CRACKLIB_DICTPATH))) {
+                                printf("Please use a different password.\n\n");
+                                printf("The one you have chosen is easily crackable:\n");
+				printf("    %s.\n", msg);
+				printf("\nHint: take a phrase you won't forget, and use the first\n");
+				printf("letter of each word in the phrase as your password.  Use\n");
+				printf("some punctuation characters as well, and you'll have a\n");
+				printf("good password.\n\n");
+                                continue;  /* go round and round until they get it right */
+                        }
+
+		}
+#endif /* CRACKLIB_DICTPATH */
+/* cracklib end */
 		if (force_mix_case) {
 		    for (t = p; *t && islower(*t); ++t);
 		    if (!*t && (uid != 0 || ++tries < 2)) {
@@ -153,6 +188,13 @@
 			    continue;
 		    }
 		}
+		/* digit? */
+		for (t = p; *t && !islower(*t); ++t);
+		if (!*t && (uid != 0 || ++tries < 2)) {
+	 	    (void)printf("Please use at lease one digit in your password.\n");
+		    continue;
+		}
+
 		(void)strcpy(buf, p);
 		if (!strcmp(buf, getpass("Retype new password:")))
 			break;
